package com.sun.corba.se.impl.oa.poa;

import java.util.Set;
import org.omg.PortableServer.POAPackage.WrongPolicy;
import org.omg.PortableServer.Servant;
import java.util.HashMap;
import java.util.Map;

public abstract class ActiveObjectMap
{
    protected POAImpl poa;
    private Map keyToEntry;
    private Map entryToServant;
    private Map servantToEntry;
    
    protected ActiveObjectMap(POAImpl poa) {
        this.keyToEntry = new HashMap();
        this.entryToServant = new HashMap();
        this.servantToEntry = new HashMap();
        this.poa = poa;
    }
    
    public static ActiveObjectMap create(POAImpl poaImpl, boolean b) {
        if (b) {
            return new MultipleObjectMap(poaImpl);
        }
        return new SingleObjectMap(poaImpl);
    }
    
    public final boolean contains(Servant servant) {
        return this.servantToEntry.containsKey(servant);
    }
    
    public final boolean containsKey(ActiveObjectMap.Key key) {
        return this.keyToEntry.containsKey(key);
    }
    
    public final AOMEntry get(ActiveObjectMap.Key key) {
        AOMEntry aomEntry = (AOMEntry)this.keyToEntry.get(key);
        if (aomEntry == null) {
            aomEntry = new AOMEntry(this.poa);
            this.putEntry(key, aomEntry);
            return aomEntry;
        }
        return aomEntry;
    }
    
    public final Servant getServant(AOMEntry aomEntry) {
        return (Servant)this.entryToServant.get(aomEntry);
    }
    
    public abstract ActiveObjectMap.Key getKey(AOMEntry p0) throws WrongPolicy;
    
    public ActiveObjectMap.Key getKey(Servant servant) throws WrongPolicy {
        return this.getKey((AOMEntry)this.servantToEntry.get(servant));
    }
    
    protected void putEntry(ActiveObjectMap.Key key, AOMEntry aomEntry) {
        this.keyToEntry.put(key, aomEntry);
    }
    
    public final void putServant(Servant servant, AOMEntry aomEntry) {
        this.entryToServant.put(aomEntry, servant);
        this.servantToEntry.put(servant, aomEntry);
    }
    
    protected abstract void removeEntry(AOMEntry p0, ActiveObjectMap.Key p1);
    
    public final void remove(ActiveObjectMap.Key key) {
        AOMEntry aomEntry = (AOMEntry)this.keyToEntry.remove(key);
        Servant servant = (Servant)this.entryToServant.remove(aomEntry);
        if (servant != null) {
            this.servantToEntry.remove(servant);
        }
        this.removeEntry(aomEntry, key);
    }
    
    public abstract boolean hasMultipleIDs(AOMEntry p0);
    
    protected void clear() {
        this.keyToEntry.clear();
    }
    
    public final Set keySet() {
        return this.keyToEntry.keySet();
    }
    
    public static class Key
    {
        public byte[] id;
        
        Key(byte[] id) {
            this.id = id;
        }
        
        @Override
        public String toString() {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < this.id.length; ++i) {
                sb.append(Integer.toString((int)this.id[i], 16));
                if (i != this.id.length - 1) {
                    sb.append(":");
                }
            }
            return sb.toString();
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof ActiveObjectMap.Key)) {
                return false;
            }
            ActiveObjectMap.Key key = (ActiveObjectMap.Key)o;
            if (key.id.length != this.id.length) {
                return false;
            }
            int i = 0;
            while (i < this.id.length) {
                if (this.id[i] != key.id[i]) {
                    return false;
                }
                ++i;
            }
            return true;
        }
        
        @Override
        public int hashCode() {
            int n = 0;
            for (int i = 0; i < this.id.length; ++i) {
                n = 31 * n + this.id[i];
            }
            return n;
        }
    }
}
